@@ -1,4 +1,4 @@
-"""`Datajudge` allows to assess  whether data from database complies with reference
+"""datajudge allows to assess  whether data from database complies with reference
 information."""
 
 
@@ -19,4 +19,4 @@ __all__ = [
 try:
     __version__ = pkg_resources.get_distribution(__name__).version
 except Exception:
-    __version__ = "1.1.1"
+    __version__ = "1.2.0"
