@@ -9,6 +9,7 @@ from dataclasses import dataclass
 from typing import Callable, Sequence, final, overload
 
 import sqlalchemy as sa
+from sqlalchemy.sql import selectable
 from sqlalchemy.sql.expression import FromClause
 
 
@@ -24,10 +25,46 @@ def is_snowflake(engine: sa.engine.Engine) -> bool:
     return engine.name == "snowflake"
 
 
+def is_bigquery(engine: sa.engine.Engine) -> bool:
+    return engine.name == "bigquery"
+
+
 def get_table_columns(table, column_names):
     return [table.c[column_name] for column_name in column_names]
 
 
+def apply_patches(engine: sa.engine.Engine):
+    """
+    Apply patches to e.g. specific dialect not implemented by sqlalchemy
+    """
+
+    if is_bigquery(engine):
+        # Patch for the EXCEPT operator (see BigQuery set operators
+        # https://cloud.google.com/bigquery/docs/reference/standard-sql/query-syntax#set_operators)
+        # This is implemented in the same way as for sqlalchemy-bigquery, see
+        # https://github.com/googleapis/python-bigquery-sqlalchemy/blob/f1889443bd4d680550387b9bb14daeea8eb792d4/sqlalchemy_bigquery/base.py#L187
+        compound_keywords_extensions = {
+            selectable.CompoundSelect.EXCEPT: "EXCEPT DISTINCT",
+            selectable.CompoundSelect.EXCEPT_ALL: "EXCEPT ALL",
+        }
+        engine.dialect.statement_compiler.compound_keywords.update(
+            compound_keywords_extensions
+        )
+
+        # Patch for the INTERSECT operator (see BigQuery set operators
+        # https://cloud.google.com/bigquery/docs/reference/standard-sql/query-syntax#set_operators)
+        # This might cause some problems (see discussion in
+        # https://github.com/googleapis/python-bigquery-sqlalchemy/issues/388) but doesn't seem
+        # to be an issue here.
+        compound_keywords_extensions = {
+            selectable.CompoundSelect.INTERSECT: "INTERSECT DISTINCT",
+            selectable.CompoundSelect.INTERSECT_ALL: "INTERSECT ALL",
+        }
+        engine.dialect.statement_compiler.compound_keywords.update(
+            compound_keywords_extensions
+        )
+
+
 @overload
 def lowercase_column_names(column_names: str) -> str:
     ...
@@ -288,7 +325,13 @@ class DataReference:
                 f"Trying to access column of DataReference "
                 f"{self.get_string()} yet none is given."
             )
-        return self.get_columns(engine)[0]
+        columns = self.get_columns(engine)
+        if len(columns) > 1:
+            raise ValueError(
+                "DataReference was expected to only have a single column but had multiple: "
+                f"{columns}"
+            )
+        return columns[0]
 
     def get_columns(self, engine):
         """Fetch all relevant columns of a DataReference."""
@@ -306,7 +349,7 @@ class DataReference:
     def get_column_selection_string(self):
         if self.columns is None:
             return " * "
-        return ", ".join(self.columns)
+        return ", ".join(map(lambda x: f"'{x}'", self.columns))
 
     def get_clause_string(self, *, return_where=True):
         where_string = "WHERE " if return_where else ""
@@ -315,7 +358,7 @@ class DataReference:
     def get_string(self):
         if self.columns is None:
             return str(self.data_source)
-        return f"{self.data_source}'s columns " f" {self.get_column_selection_string()}"
+        return f"{self.data_source}'s column(s) {self.get_column_selection_string()}"
 
 
 def merge_conditions(condition1, condition2):
@@ -355,6 +398,16 @@ def get_date_span(engine, ref, date_column_name):
                 )
             ]
         )
+    elif is_bigquery(engine):
+        selection = sa.select(
+            [
+                sa.func.date_diff(
+                    sa.func.max(column),
+                    sa.func.min(column),
+                    sa.literal_column("DAY"),
+                )
+            ]
+        )
     else:
         raise NotImplementedError(
             "Date spans not yet implemented for this sql dialect."
@@ -377,7 +430,7 @@ def get_date_growth_rate(engine, ref, ref2, date_column, date_column2):
     date_span2, selections2 = get_date_span(engine, ref2, date_column2)
     if date_span2 == 0:
         raise ValueError("Reference date span is not allowed to be zero.")
-    return date_span / date_span2 - 1, [*selections, *selections]
+    return date_span / date_span2 - 1, [*selections, *selections2]
 
 
 def get_date_overlaps_nd(
@@ -534,17 +587,13 @@ def get_date_gaps(
 
     start_rank_column = (
         sa.func.row_number()
-        .over(
-            order_by=raw_start_table.c[start_column],
-        )
+        .over(order_by=[raw_start_table.c[col] for col in [start_column] + key_columns])
         .label("start_rank")
     )
 
     end_rank_column = (
         sa.func.row_number()
-        .over(
-            order_by=raw_end_table.c[end_column],
-        )
+        .over(order_by=[raw_end_table.c[col] for col in [end_column] + key_columns])
         .label("end_rank")
     )
 
@@ -571,6 +620,16 @@ def get_date_gaps(
             )
             > legitimate_gap_size
         )
+    elif is_bigquery(engine):
+        # see https://cloud.google.com/bigquery/docs/reference/standard-sql/date_functions#date_diff
+        # Note that to have a gap (positive date_diff), the first date (start table)
+        # in date_diff must be greater than the second date (end_table)
+        gap_condition = (
+            sa.func.date_diff(
+                start_table.c[start_column], end_table.c[end_column], sa.text("DAY")
+            )
+            > legitimate_gap_size
+        )
     elif is_postgresql(engine):
         gap_condition = (start_table.c[start_column] > end_table.c[end_column] + 1,)
         gap_condition = (
@@ -780,7 +839,9 @@ def get_column_type(engine, ref):
         if column_type == "DECIMAL(38, 0)":
             column_type = "integer"
         return column_type, None
-    column_type = [str(column.type).split(" ")[0] for column in table.columns][0]
+    column_type = [
+        str(column.type).split(" ", maxsplit=1)[0] for column in table.columns
+    ][0]
     return column_type, None
 
 
@@ -904,87 +965,193 @@ def get_column_array_agg(
     return result, selections
 
 
+def _cdf_selection(engine, ref: DataReference, cdf_label: str, value_label: str):
+    """Create an empirical cumulative distribution function values.
+
+    Concretely, create a selection with values from ``value_label`` as well as
+    the empirical cumulative didistribution function values, labeled as
+    ``cdf_label``.
+    """
+    col = ref.get_column(engine)
+    selection = ref.get_selection(engine).subquery()
+
+    # Step 1: Calculate the CDF over the value column.
+    cdf_selection = sa.select(
+        [
+            selection.c[col].label(value_label),
+            sa.func.cume_dist().over(order_by=col).label(cdf_label),
+        ]
+    ).subquery()
+
+    # Step 2: Aggregate rows s.t. every value occurs only once.
+    grouped_cdf_selection = (
+        sa.select(
+            [
+                cdf_selection.c[value_label],
+                sa.func.max(cdf_selection.c[cdf_label]).label(cdf_label),
+            ]
+        )
+        .group_by(cdf_selection.c[value_label])
+        .subquery()
+    )
+    return grouped_cdf_selection
+
+
+def _cross_cdf_selection(
+    engine, ref1: DataReference, ref2: DataReference, cdf_label: str, value_label: str
+):
+    """Create a cross cumulative distribution function selection given two samples.
+
+    Concretely, both ``DataReference``s are expected to have specified a single relevant column.
+    This function will generate a selection with rows of the kind ``(value, cdf1(value), cdf2(value))``,
+    where ``cdf1`` is the cumulative distribution function of ``ref1`` and ``cdf2`` of ``ref2``.
+
+    E.g. if ``ref`` is a reference to a table's column with values ``[1, 1, 3, 2]``, and ``ref2`` is
+    a reference to a table's column with values ``[2, 5, 4]``, executing the returned selection should
+    yield a table of the following kind: ``[(1, .5, 0), (2, .75, 1/3), (3, 1 ,1/3), (4, 1, 2/3), (5, 1, 1)]``.
+    """
+    cdf_label1 = cdf_label + "1"
+    cdf_label2 = cdf_label + "2"
+    group_label1 = "_grp1"
+    group_label2 = "_grp2"
+
+    cdf_selection1 = _cdf_selection(engine, ref1, cdf_label, value_label)
+    cdf_selection2 = _cdf_selection(engine, ref2, cdf_label, value_label)
+
+    # Step 3: Combine the cdfs.
+    cross_cdf = (
+        sa.select(
+            sa.func.coalesce(
+                cdf_selection1.c[value_label], cdf_selection2.c[value_label]
+            ).label(value_label),
+            cdf_selection1.c[cdf_label].label(cdf_label1),
+            cdf_selection2.c[cdf_label].label(cdf_label2),
+        )
+        .select_from(
+            cdf_selection1.join(
+                cdf_selection2,
+                cdf_selection1.c[value_label] == cdf_selection2.c[value_label],
+                isouter=True,
+                full=True,
+            )
+        )
+        .subquery()
+    )
+
+    def _cdf_index_column(table, value_label, cdf_label, group_label):
+        return (
+            sa.func.count(table.c[cdf_label])
+            .over(order_by=table.c[value_label])
+            .label(group_label)
+        )
+
+    # Step 4: Create a grouper id based on the value count; this is just a helper for forward-filling.
+    # In other words, we point rows to their most recent present value - per sample. This is necessary
+    # Due to the nature of the full outer join.
+    indexed_cross_cdf = sa.select(
+        [
+            cross_cdf.c[value_label],
+            _cdf_index_column(cross_cdf, value_label, cdf_label1, group_label1),
+            cross_cdf.c[cdf_label1],
+            _cdf_index_column(cross_cdf, value_label, cdf_label2, group_label2),
+            cross_cdf.c[cdf_label2],
+        ]
+    ).subquery()
+
+    def _forward_filled_cdf_column(table, cdf_label, value_label, group_label):
+        return (
+            # Step 6: Replace NULL values at the beginning with 0 to enable computation of difference.
+            sa.func.coalesce(
+                (
+                    # Step 5: Forward-Filling: Select first non-NULL value per group (defined in the prev. step).
+                    sa.func.first_value(table.c[cdf_label]).over(
+                        partition_by=table.c[group_label], order_by=table.c[value_label]
+                    )
+                ),
+                0,
+            ).label(cdf_label)
+        )
+
+    filled_cross_cdf = sa.select(
+        [
+            indexed_cross_cdf.c[value_label],
+            _forward_filled_cdf_column(
+                indexed_cross_cdf, cdf_label1, value_label, group_label1
+            ),
+            _forward_filled_cdf_column(
+                indexed_cross_cdf, cdf_label2, value_label, group_label2
+            ),
+        ]
+    )
+    return filled_cross_cdf, cdf_label1, cdf_label2
+
+
 def get_ks_2sample(
-    engine: sa.engine.Engine, table1: tuple, table2: tuple
-) -> tuple[float, int, int]:
+    engine: sa.engine.Engine,
+    ref1: DataReference,
+    ref2: DataReference,
+):
     """
-    Runs the query for the two-sample Kolmogorov-Smirnov test and returns the test statistic d.
+    Run the query for the two-sample Kolmogorov-Smirnov test and return the test statistic d.
+
+    For a raw-sql version of this query, please see this PR:
+    https://github.com/Quantco/datajudge/pull/28/
     """
+    cdf_label = "cdf"
+    value_label = "val"
+    filled_cross_cdf_selection, cdf_label1, cdf_label2 = _cross_cdf_selection(
+        engine, ref1, ref2, cdf_label, value_label
+    )
 
-    # make sure we have a string representation here
-    table1_selection, col1 = str(table1[0]), str(table1[1])
-    table2_selection, col2 = str(table2[0]), str(table2[1])
+    filled_cross_cdf = filled_cross_cdf_selection.subquery()
 
-    if is_mssql(engine):  # "tempdb.dbo".table_name -> tempdb.dbo.table_name
-        table1_selection = table1_selection.replace('"', "")
-        table2_selection = table2_selection.replace('"', "")
+    # Step 7: Calculate final statistic: maximal distance.
+    final_selection = sa.select(
+        sa.func.max(
+            sa.func.abs(filled_cross_cdf.c[cdf_label1] - filled_cross_cdf.c[cdf_label2])
+        )
+    )
 
-    # for RawQueryDataSource this could be a whole subquery and will therefore need to be wrapped
-    if "SELECT" in table1_selection:
-        table1_selection = f"({table1_selection})"
-        table2_selection = f"({table2_selection})"
+    with engine.connect() as connection:
+        d_statistic = connection.execute(final_selection).scalar()
 
-    # for a more extensive explanation, see:
-    # https://github.com/Quantco/datajudge/pull/28#issuecomment-1165587929
-    ks_query_string = f"""
-        WITH
-        tab1 AS ( -- Step 0: Prepare data source and value column
-            SELECT {col1} as val FROM {table1_selection}
-        ),
-        tab2 AS (
-            SELECT {col2} as val FROM {table2_selection}
-        ),
-        tab1_cdf AS ( -- Step 1: Calculate the CDF over the value column
-            SELECT val, cume_dist() over (order by val) as cdf
-            FROM tab1
-        ),
-        tab2_cdf AS (
-            SELECT val, cume_dist() over (order by val) as cdf
-            FROM tab2
-        ),
-        tab1_grouped AS ( -- Step 2: Remove unnecessary values, s.t. we have (x, cdf(x)) rows only
-            SELECT val, MAX(cdf) as cdf
-            FROM tab1_cdf
-            GROUP BY val
-        ),
-        tab2_grouped AS (
-            SELECT val, MAX(cdf) as cdf
-            FROM tab2_cdf
-            GROUP BY val
-        ),
-        joined_cdf AS ( -- Step 3: combine the cdfs
-            SELECT coalesce(tab1_grouped.val, tab2_grouped.val) as v, tab1_grouped.cdf as cdf1, tab2_grouped.cdf as cdf2
-            FROM tab1_grouped FULL OUTER JOIN tab2_grouped ON tab1_grouped.val = tab2_grouped.val
-        ),
-        -- Step 4: Create a grouper id based on the value count; this is just a helper for forward-filling
-        grouped_cdf AS (
-            SELECT v,
-                COUNT(cdf1) over (order by v) as _grp1,
-                cdf1,
-                COUNT(cdf2) over (order by v) as _grp2,
-                cdf2
-            FROM joined_cdf
-        ),
-        -- Step 5: Forward-Filling: Select first non-null value per group (defined in the prev. step)
-        filled_cdf AS (
-            SELECT v,
-                first_value(cdf1) over (partition by _grp1 order by v) as cdf1_filled,
-                first_value(cdf2) over (partition by _grp2 order by v) as cdf2_filled
-            FROM grouped_cdf),
-        -- Step 6: Replace NULL values (at the beginning) with 0 to calculate difference
-        replaced_nulls AS (
-            SELECT coalesce(cdf1_filled, 0) as cdf1, coalesce(cdf2_filled, 0) as cdf2
-            FROM filled_cdf)
-        -- Step 7: Calculate final statistic as max. distance
-        SELECT MAX(ABS(cdf1 - cdf2)) FROM replaced_nulls;
-    """
+    return d_statistic, [final_selection]
 
-    d_statistic = engine.execute(ks_query_string).scalar()
-    n_samples = engine.execute(
-        f"SELECT COUNT(*) FROM {table1_selection} as n_table"
-    ).scalar()
-    m_samples = engine.execute(
-        f"SELECT COUNT(*) FROM {table2_selection} as m_table"
-    ).scalar()
 
-    return d_statistic, n_samples, m_samples
+def get_regex_violations(engine, ref, aggregated, regex, n_counterexamples):
+    subquery = ref.get_selection(engine)
+    column = ref.get_column(engine)
+    if aggregated:
+        subquery = subquery.distinct()
+    subquery = subquery.subquery()
+    violation_selection = sa.select(subquery.c[column]).where(
+        sa.not_(subquery.c[column].regexp_match(regex))
+    )
+    n_violations_selection = sa.select([sa.func.count()]).select_from(
+        violation_selection.subquery()
+    )
+
+    selections = [n_violations_selection]
+
+    if n_counterexamples == -1:
+        counterexamples_selection = violation_selection
+    elif n_counterexamples == 0:
+        counterexamples_selection = None
+    elif n_counterexamples > 0:
+        counterexamples_selection = violation_selection.limit(n_counterexamples)
+    else:
+        raise ValueError(f"Unexpected number of counterexamples: {n_counterexamples}")
+
+    if counterexamples_selection is not None:
+        selections.append(counterexamples_selection)
+
+    with engine.connect() as connection:
+        n_violations_result = connection.execute(n_violations_selection).scalar()
+        if counterexamples_selection is None:
+            counterexamples = []
+        else:
+            counterexamples_result = connection.execute(
+                counterexamples_selection
+            ).fetchall()
+            counterexamples = [result[0] for result in counterexamples_result]
+    return (n_violations_result, counterexamples), selections
