@@ -7,7 +7,7 @@ import urllib.parse
 import pytest
 import sqlalchemy as sa
 
-from datajudge.db_access import is_mssql
+from datajudge.db_access import apply_patches, is_bigquery, is_mssql
 
 TEST_DB_NAME = "tempdb"
 SCHEMA = "dbo"  # 'dbo' is the standard schema in mssql
@@ -33,15 +33,21 @@ def get_engine(backend) -> sa.engine.Engine:
         password = os.environ.get("SNOWFLAKE_PASSWORD")
         account = os.environ.get("SNOWFLAKE_ACCOUNT", "")
         connection_string = f"snowflake://{user}:{password}@{account}/datajudge/DBO?warehouse=datajudge&role=accountadmin"
+    elif "bigquery" in backend:
+        gcp_project = os.environ.get("GOOGLE_CLOUD_PROJECT", "scratch-361908")
+        connection_string = f"bigquery://{gcp_project}"
 
-    return sa.create_engine(connection_string)
+    engine = sa.create_engine(connection_string, echo=True)
+    apply_patches(engine)
+
+    return engine
 
 
 @pytest.fixture(scope="module")
 def engine(backend):
     engine = get_engine(backend)
     with engine.connect() as conn:
-        if engine.name == "postgresql":
+        if engine.name in ("postgresql", "bigquery"):
             conn.execute(f"CREATE SCHEMA IF NOT EXISTS {SCHEMA}")
     return engine
 
@@ -52,8 +58,9 @@ def metadata():
 
 
 def _handle_table(engine, metadata, table_name, columns, data):
-    engine.execute(f"DROP TABLE IF EXISTS {SCHEMA}.{table_name}")
     with engine.connect() as conn:
+        if sa.inspect(conn).has_table(table_name, schema=SCHEMA):
+            return
         table = sa.Table(table_name, metadata, *columns, schema=SCHEMA)
         table.create(conn)
         conn.execute(table.insert().values(), data)
@@ -665,8 +672,14 @@ def groupby_aggregation_table_incorrect(engine, metadata):
 @pytest.fixture(scope="module")
 def random_normal_table(engine, metadata):
     """
-    Table containing 10_000 randomly distributed values with mean = 0 and std.dev = 1.
+    Table with normally distributed values of varying means and sd 1.
     """
+
+    if is_bigquery(engine):
+        # It takes too long to insert the table into BigQuery,
+        # test using this fixture must be disabled for BigQuery
+        return None, None, None
+
     table_name = "random_normal_table"
     columns = [
         sa.Column("value_0_1", sa.Float()),
@@ -700,12 +713,19 @@ def capitalization_table(engine, metadata):
     lowercase_column = "num_employees"
     # Create and populate table with raw strings as to ensure
     # the columns are actually created with capitalization.
-    str_datatype = "VARCHAR(20)" if is_mssql(engine) else "TEXT"
+    primary_key = "PRIMARY KEY"
+    if is_mssql(engine):
+        str_datatype = "VARCHAR(20)"
+    elif is_bigquery(engine):
+        str_datatype = "STRING"
+        primary_key = ""  # there is no primary key in BigQuery
+    else:
+        str_datatype = "TEXT"
     with engine.connect() as connection:
         connection.execute(f"DROP TABLE IF EXISTS {SCHEMA}.{table_name}")
         connection.execute(
             f"CREATE TABLE {SCHEMA}.{table_name} "
-            "(id INTEGER PRIMARY KEY, "
+            f"(id INTEGER {primary_key}, "
             f"{uppercase_column} {str_datatype}, {lowercase_column} INTEGER)"
         )
         connection.execute(
@@ -715,10 +735,32 @@ def capitalization_table(engine, metadata):
     return TEST_DB_NAME, SCHEMA, table_name, uppercase_column, lowercase_column
 
 
+@pytest.fixture(scope="module")
+def cross_cdf_table1(engine, metadata):
+    table_name = "cross_cdf_table1"
+    col_name = "col_int"
+    columns = [sa.Column(col_name, sa.Integer())]
+    col_values = [1, 1, 3, 2]
+    data = [{col_name: col_value} for col_value in col_values]
+    _handle_table(engine, metadata, table_name, columns, data)
+    return TEST_DB_NAME, SCHEMA, table_name
+
+
+@pytest.fixture(scope="module")
+def cross_cdf_table2(engine, metadata):
+    table_name = "cross_cdf_table2"
+    col_name = "col_int"
+    columns = [sa.Column(col_name, sa.Integer())]
+    col_values = [3, 5, 4, 5, 8]
+    data = [{col_name: col_value} for col_value in col_values]
+    _handle_table(engine, metadata, table_name, columns, data)
+    return TEST_DB_NAME, SCHEMA, table_name
+
+
 def pytest_addoption(parser):
     parser.addoption(
         "--backend",
-        choices=(("mssql", "mssql-freetds", "postgres", "snowflake")),
+        choices=(("mssql", "mssql-freetds", "postgres", "snowflake", "bigquery")),
         help="which database backend to use to run the integration tests",
     )
 
