@@ -3,11 +3,9 @@ import functools
 import pytest
 
 import datajudge.requirements as requirements
-from datajudge.constraints.stats import KolmogorovSmirnov2Sample
 from datajudge.db_access import (
     Condition,
-    DataReference,
-    TableDataSource,
+    is_bigquery,
     is_mssql,
     is_postgresql,
     is_snowflake,
@@ -16,7 +14,7 @@ from datajudge.db_access import (
 
 def skip_if_mssql(engine):
     if is_mssql(engine):
-        pytest.skip("array_agg not supported by SQL Server")
+        pytest.skip("functionality not supported by SQL Server")
 
 
 def identity(boolean_value):
@@ -1258,6 +1256,7 @@ def test_date_no_gap_within_inclusion_exclusion(engine, date_table_gap, data):
     assert operation(test_result.outcome), test_result.failure_message
 
 
+@pytest.mark.parametrize("computation_in_db", [True, False])
 @pytest.mark.parametrize(
     "data",
     [
@@ -1265,11 +1264,26 @@ def test_date_no_gap_within_inclusion_exclusion(engine, date_table_gap, data):
         (negation, "^hi[0-9]$", None),
     ],
 )
-def test_varchar_regex_within(engine, mix_table1, data):
+def test_varchar_regex_within(engine, mix_table1, computation_in_db, data):
     (operation, regex, condition) = data
     req = requirements.WithinRequirement.from_table(*mix_table1)
-    req.add_varchar_regex_constraint("col_varchar", regex, condition=condition)
-    assert operation(req[0].test(engine).outcome)
+    if computation_in_db:
+        # bigquery dialect does not support regular expressions (sqlalchemy-bigquery 1.4.4)
+        if is_mssql(engine) or is_bigquery(engine):
+            pytest.skip("Functionality not supported by given dialect.")
+        req.add_varchar_regex_constraint_db(
+            column="col_varchar",
+            regex=regex,
+            condition=condition,
+        )
+    else:
+        req.add_varchar_regex_constraint(
+            column="col_varchar",
+            regex=regex,
+            condition=condition,
+        )
+    test_result = req[0].test(engine)
+    assert operation(test_result.outcome), test_result.failure_message
 
 
 @pytest.mark.parametrize(
@@ -1288,9 +1302,11 @@ def test_varchar_regex_with_none_within(engine, varchar_table1, data):
         condition=condition,
         allow_none=allow,
     )
-    assert operation(req[0].test(engine).outcome)
+    test_result = req[0].test(engine)
+    assert operation(test_result.outcome), test_result.failure_message
 
 
+@pytest.mark.parametrize("computation_in_db", [True, False])
 @pytest.mark.parametrize(
     "data",
     [
@@ -1300,23 +1316,135 @@ def test_varchar_regex_with_none_within(engine, varchar_table1, data):
         (negation, None, False, 0.15),
     ],
 )
-def test_varchar_regex_tolerance(engine, varchar_table_real, data):
+def test_varchar_regex_tolerance(engine, varchar_table_real, computation_in_db, data):
     (operation, condition, aggregated, tolerance) = data
     req = requirements.WithinRequirement.from_table(*varchar_table_real)
-    req.add_varchar_regex_constraint(
-        "col_varchar",
-        r"[A-Z][0-9]{2}\.[0-9]{0,2}$",
-        condition=condition,
-        relative_tolerance=tolerance,
-        aggregated=aggregated,
-    )
-    assert operation(req[0].test(engine).outcome)
+    if computation_in_db:
+        # TODO: This feature is available in snowflake-sqlalchemy 1.4.0.
+        # Once we remove or update the pinned version, we can enable this test
+        # for snowflake.
+        # The feature is not supported in sqlalchemy-bigquery 1.4.4
+        if is_mssql(engine) or is_snowflake(engine) or is_bigquery(engine):
+            pytest.skip("Functionality not supported by given dialect.")
+        req.add_varchar_regex_constraint_db(
+            "col_varchar",
+            r"[A-Z][0-9]{2}\.[0-9]{0,2}$",
+            condition=condition,
+            relative_tolerance=tolerance,
+            aggregated=aggregated,
+        )
+    else:
+        req.add_varchar_regex_constraint(
+            "col_varchar",
+            r"[A-Z][0-9]{2}\.[0-9]{0,2}$",
+            condition=condition,
+            relative_tolerance=tolerance,
+            aggregated=aggregated,
+        )
+    test_result = req[0].test(engine)
+    assert operation(test_result.outcome), test_result.failure_message
+
+
+@pytest.mark.parametrize("computation_in_db", [True, False])
+@pytest.mark.parametrize(
+    "n_counterexamples, n_received_counterexamples",
+    [
+        (-1, 2),
+        (0, 0),
+        (1, 1),
+        (2, 2),
+        (3, 2),
+    ],
+)
+def test_varchar_regex_counterexample(
+    engine,
+    varchar_table_real,
+    computation_in_db,
+    n_counterexamples,
+    n_received_counterexamples,
+):
+    req = requirements.WithinRequirement.from_table(*varchar_table_real)
+    if computation_in_db:
+        # TODO: This feature is available in snowflake-sqlalchemy 1.4.0.
+        # Once we remove or update the pinned version, we can enable this test
+        # for snowflake.
+        # The feature is not supported in sqlalchemy-bigquery 1.4.4
+        if is_mssql(engine) or is_snowflake(engine) or is_bigquery(engine):
+            pytest.skip("Functionality not supported by given dialect.")
+        req.add_varchar_regex_constraint_db(
+            "col_varchar",
+            r"[A-Z][0-9]{2}\.[0-9]{0,2}$",
+            condition=None,
+            relative_tolerance=0,
+            aggregated=True,
+            n_counterexamples=n_counterexamples,
+        )
+    else:
+        req.add_varchar_regex_constraint(
+            "col_varchar",
+            r"[A-Z][0-9]{2}\.[0-9]{0,2}$",
+            condition=None,
+            relative_tolerance=0,
+            aggregated=True,
+            n_counterexamples=n_counterexamples,
+        )
+    test_result = req[0].test(engine)
+    failure_message = test_result.failure_message
+    # If no counterexample are given, this marker should not be present in the
+    # failure message.
+    marker = "Some counterexamples consist of the following: "
+    location = failure_message.find(marker)
+    if n_received_counterexamples == 0:
+        assert location == -1
+    else:
+        assert location != -1
+        # In the example of this very fixture, we know that no commas are used
+        # in values. We can therefore assume that commas indicate seperation
+        # between counterexamples.
+        assert (
+            len(failure_message[location + len(marker) :].split(","))
+            == n_received_counterexamples
+        )
+
+
+@pytest.mark.parametrize("computation_in_db", [True, False])
+@pytest.mark.parametrize("n_counterexamples", [-2, -100])
+def test_varchar_regex_counterexample_invalid(
+    engine, varchar_table_real, computation_in_db, n_counterexamples
+):
+    req = requirements.WithinRequirement.from_table(*varchar_table_real)
+    if computation_in_db:
+        # TODO: This feature is available in snowflake-sqlalchemy 1.4.0.
+        # Once we remove or update the pinned version, we can enable this test
+        # for snowflake.
+        # The feature is not supported in sqlalchemy-bigquery 1.4.4
+        if is_mssql(engine) or is_snowflake(engine) or is_bigquery(engine):
+            pytest.skip("Functionality not supported by given dialect.")
+        req.add_varchar_regex_constraint_db(
+            "col_varchar",
+            r"[A-Z][0-9]{2}\.[0-9]{0,2}$",
+            condition=None,
+            relative_tolerance=0,
+            aggregated=True,
+            n_counterexamples=n_counterexamples,
+        )
+    else:
+        req.add_varchar_regex_constraint(
+            "col_varchar",
+            r"[A-Z][0-9]{2}\.[0-9]{0,2}$",
+            condition=None,
+            relative_tolerance=0,
+            aggregated=True,
+            n_counterexamples=n_counterexamples,
+        )
+    with pytest.raises(ValueError):
+        req[0].test(engine)
 
 
 def test_backend_dependent_condition(engine, mix_table1):
     if is_mssql(engine):
         condition = Condition(raw_string="DATALENGTH(col_varchar) = 3")
-    elif is_postgresql(engine) or is_snowflake(engine):
+    elif is_postgresql(engine) or is_snowflake(engine) or is_bigquery(engine):
         condition = Condition(raw_string="LENGTH(col_varchar) = 3")
     else:
         raise NotImplementedError(f"Unexpected backend: {engine.name}")
@@ -1472,6 +1600,9 @@ def test_column_superset_between(engine, get_fixture, data):
     ],
 )
 def test_primary_key_definition_within(engine, pk_table, data):
+    if is_bigquery(engine):
+        pytest.skip("No primary key concept in BigQuery")
+
     (operation, columns) = data
     req = requirements.WithinRequirement.from_table(*pk_table)
     req.add_primary_key_definition_constraint(columns)
@@ -1501,9 +1632,10 @@ def test_uniqueness_within(engine, mix_table2, data):
         max_absolute_n_duplicates,
     ) = data
     # For an unknown reason, the condition `Condition(raw_string="col_int % 2 = 0")`
-    # is not correctly compiled when dealing with snowflake.
+    # is not correctly compiled when dealing with snowflake or bigquery.
+    # Use the mod function instead
     if (
-        is_snowflake(engine)
+        (is_snowflake(engine) or is_bigquery(engine))
         and condition is not None
         and condition.raw_string is not None
         and "% 2 = 0" in condition.raw_string
@@ -1761,32 +1893,72 @@ def test_groupby_aggregation_within_with_failures(
     assert operation(test_result.outcome), test_result.failure_message
 
 
-def test_diff_average_between():
-    return
-
-
 @pytest.mark.parametrize(
     "data",
     [
-        (identity, "col_int", "col_int", None, 1.0),
-        (identity, "col_int", "col_int", Condition("col_int >= 3"), 1.0),
+        (identity, "col_int", "col_int", None, None, 1.0),
+        (
+            identity,
+            "col_int",
+            "col_int",
+            Condition(raw_string="col_int >= 3"),
+            Condition(raw_string="col_int >= 3"),
+            1.0,
+        ),
     ],
 )
 def test_ks_2sample_constraint_perfect_between(engine, int_table1, data):
     """
     Test Kolmogorov-Smirnov for the same column -> p-value should be perfect 1.0.
     """
-    (operation, col_1, col_2, condition, significance_level) = data
+    (operation, col_1, col_2, condition1, condition2, significance_level) = data
     req = requirements.BetweenRequirement.from_tables(*int_table1, *int_table1)
     req.add_ks_2sample_constraint(
         column1=col_1,
         column2=col_2,
-        condition1=condition,
-        condition2=condition,
+        condition1=condition1,
+        condition2=condition2,
         significance_level=significance_level,
     )
+    test_result = req[0].test(engine)
+    assert operation(test_result.outcome), test_result.failure_message
 
-    assert operation(req[0].test(engine).outcome)
+
+@pytest.mark.parametrize(
+    "condition1, condition2",
+    [
+        (
+            None,
+            Condition(raw_string="col_int >= 10"),
+        ),
+        (
+            Condition(raw_string="col_int >= 10"),
+            None,
+        ),
+        (
+            Condition(raw_string="col_int >= 10"),
+            Condition(raw_string="col_int >= 3"),
+        ),
+    ],
+)
+def test_ks_2sample_constraint_perfect_between_different_conditions(
+    engine, int_table1, condition1, condition2
+):
+    """
+    Test Kolmogorov-Smirnov for the same column but different conditions.
+    As a consequence, since the data is distinct, the tests are expected
+    to fail for a very high significance level.
+    """
+    req = requirements.BetweenRequirement.from_tables(*int_table1, *int_table1)
+    req.add_ks_2sample_constraint(
+        column1="col_int",
+        column2="col_int",
+        condition1=condition1,
+        condition2=condition2,
+        significance_level=1.0,
+    )
+    test_result = req[0].test(engine)
+    assert negation(test_result.outcome), test_result.failure_message
 
 
 @pytest.mark.parametrize(
@@ -1804,8 +1976,8 @@ def test_ks_2sample_constraint_wrong_between(
     req.add_ks_2sample_constraint(
         column1=col_1, column2=col_2, significance_level=min_p_value
     )
-
-    assert operation(req[0].test(engine).outcome)
+    test_result = req[0].test(engine)
+    assert operation(test_result.outcome), test_result.failure_message
 
 
 @pytest.mark.parametrize(
@@ -1827,6 +1999,10 @@ def test_ks_2sample_constraint_wrong_between(
     ],
 )
 def test_ks_2sample_random(engine, random_normal_table, configuration):
+
+    if is_bigquery(engine):
+        pytest.skip("It takes too long to insert the table into BigQuery")
+
     (operation, col_1, col_2, min_p_value) = configuration
     req = requirements.BetweenRequirement.from_tables(
         *random_normal_table, *random_normal_table
@@ -1835,45 +2011,4 @@ def test_ks_2sample_random(engine, random_normal_table, configuration):
         column1=col_1, column2=col_2, significance_level=min_p_value
     )
     test_result = req[0].test(engine)
-    assert operation(test_result.outcome)
-
-
-@pytest.mark.parametrize(
-    "configuration",
-    [  # these values were calculated using scipy.stats.ks_2samp on scipy=1.8.1
-        ("value_0_1", "value_0_1", 0.0, 1.0),
-        ("value_0_1", "value_005_1", 0.0294, 0.00035221594346540835),
-        ("value_0_1", "value_02_1", 0.0829, 2.6408848561586672e-30),
-        ("value_0_1", "value_1_1", 0.3924, 0.0),
-    ],
-)
-def test_ks_2sample_implementation(engine, random_normal_table, configuration):
-    col_1, col_2, expected_d, expected_p = configuration
-    database, schema, table = random_normal_table
-    tds = TableDataSource(database, table, schema)
-    ref = DataReference(tds, columns=[col_1])
-    ref2 = DataReference(tds, columns=[col_2])
-
-    # retrieve table selections from data references
-    selection1 = ref.data_source.get_clause(engine)
-    column1 = ref.get_column(engine)
-    selection2 = ref2.data_source.get_clause(engine)
-    column2 = ref2.get_column(engine)
-
-    (
-        d_statistic,
-        p_value,
-        n_samples,
-        m_samples,
-    ) = KolmogorovSmirnov2Sample.calculate_statistic(
-        engine, (selection1, column1), (selection2, column2)
-    )
-
-    assert (
-        abs(d_statistic - expected_d) <= 1e-10
-    ), f"The test statistic does not match: {expected_d} vs {d_statistic}"
-
-    # 1e-05 should cover common p_values; if scipy is installed, a very accurate p_value is automatically calculated
-    assert (
-        abs(p_value - expected_p) <= 1e-05
-    ), f"The approx. p-value does not match: {expected_p} vs {p_value}"
+    assert operation(test_result.outcome), test_result.failure_message
